import AppRegistry from 'hadron-app-registry';
import store from 'stores';
import hadronApp from 'hadron-app';
import {
  validatorChanged,
  validationFetched,
  validationSaveFailed,
  validationActionChanged,
  validationLevelChanged,
  fetchSampleDocuments
} from 'modules/validation';
import { reset, INITIAL_STATE } from '../modules/index';
import javascriptStringify from 'javascript-stringify';

describe('Schema Validation Store', () => {
  const appRegistry = new AppRegistry();
  const collectionStore = {
    isReadonly: () => {
      return false;
    }
  };
  const writeStateStore = { state: { isWritable: true } };

  before(() => {
    global.hadronApp = hadronApp;
    global.hadronApp.appRegistry = appRegistry;
    global.hadronApp.appRegistry.registerStore('App.CollectionStore', collectionStore);
    global.hadronApp.appRegistry.registerStore('DeploymentAwareness.WriteStateStore', writeStateStore);
  });

  beforeEach(() => {
    store.dispatch(reset());
  });

  describe('#onActivated', () => {
    beforeEach(() => {
      store.onActivated(appRegistry);
    });

    context('when the validation changes', () => {
      it('updates the namespace in the store', (done) => {
        const unsubscribe = store.subscribe(() => {
          unsubscribe();
          expect(store.getState().fields).to.deep.equal([
            { name: 'harry',
              value: 'harry',
              score: 1,
              meta: 'field',
              version: '0.0.0' },
            { name: 'potter',
              value: 'potter',
              score: 1,
              meta: 'field',
              version: '0.0.0' }
          ]);
          done();
        });

        appRegistry.emit('fields-changed', {
          fields: {
            harry: {
              name: 'harry', path: 'harry', count: 1, type: 'Number'
            },
            potter: {
              name: 'potter', path: 'potter', count: 1, type: 'Boolean'
            }
          },
          topLevelFields: [ 'harry', 'potter' ],
          aceFields: [
            { name: 'harry',
              value: 'harry',
              score: 1,
              meta: 'field',
              version: '0.0.0' },
            { name: 'potter',
              value: 'potter',
              score: 1,
              meta: 'field',
              version: '0.0.0' }
          ]
        });
      });
    });

    context('when the data service is connected', () => {
      beforeEach(() => {
        appRegistry.emit('data-service-connected', 'error', 'ds');
      });

      it('sets the data servicein the state', () => {
        expect(store.getState().dataService.dataService).to.equal('ds');
      });

      it('sets the error in the state', () => {
        expect(store.getState().dataService.error).to.equal('error');
      });
    });
  });

  describe('#dispatch', () => {
    context('when the action is unknown', () => {
      it('returns the initial state', (done) => {
        const unsubscribe = store.subscribe(() => {
          unsubscribe();
          expect(store.getState().validation.validator).to.equal('');
          done();
        });
        store.dispatch({ type: 'UNKNOWN' });
      });
    });

    context('when the action is VALIDATOR_CHANGED', () => {
      const validator = '{ name: { $type: 4 } }';

      it('updates the validator in state', (done) => {
        const unsubscribe = store.subscribe(() => {
          unsubscribe();
          expect(store.getState().validation.validator).to.equal(validator);
          done();
        });
        store.dispatch(validatorChanged(validator));
      });
    });

    context('when the action is VALIDATION_FETCHED', () => {
      const validation = {
        validator: { name: { $type: 4 } },
        validationAction: 'warn',
        validationLevel: 'moderate'
      };

      it('updates the isLoading in state', () => {
        const isLoading = true;

        it('updates the validationAction in state', (done) => {
          const unsubscribe = store.subscribe(() => {
            unsubscribe();
            expect(store.getState().sampleDocuments.isLoading).to.equal(isLoading);
            done();
          });
          store.dispatch(fetchSampleDocuments({ matching: {}, notmatching: {}}));
        });
      });

      it('updates the validation in state if succeed', (done) => {
        const unsubscribe = store.subscribe(() => {
          const validator = javascriptStringify(validation.validator, null, 2);
          const createdValidation = {
            validator,
            validationAction: 'warn',
            validationLevel: 'moderate',
            error: null,
            syntaxError: null,
            isChanged: false,
            prevValidation: {
              validator,
              validationAction: 'warn',
              validationLevel: 'moderate'
            }
          };

          unsubscribe();
          expect(store.getState().validation).to.deep.equal(createdValidation);
          done();
        });
        store.dispatch(validationFetched(validation));
      });

      it('updates the error in state if failed', (done) => {
        const unsubscribe = store.subscribe(() => {
          const validator = javascriptStringify(validation.validator, null, 2);
          const createdValidation = {
            validator,
            validationAction: 'warn',
            validationLevel: 'moderate',
            error: { message: 'Validation fetch failed!' },
            syntaxError: null,
            isChanged: false,
            prevValidation: {
              validator,
              validationAction: 'warn',
              validationLevel: 'moderate'
            }
          };

          unsubscribe();
          expect(store.getState().validation).to.deep.equal(createdValidation);
          done();
        });
        validation.error = { message: 'Validation fetch failed!' };
        store.dispatch(validationFetched(validation));
      });
    });

    context('when the action is VALIDATION_SAVE_FAILED', () => {
      it('updates the error', (done) => {
        const unsubscribe = store.subscribe(() => {
          unsubscribe();
          expect(store.getState().validation.error).to.deep.equal({
            message: 'Validation save failed!'
          });
          done();
        });
        store.dispatch(validationSaveFailed({
          message: 'Validation save failed!'
        }));
      });
    });

    context('when the action is VALIDATION_ACTION_CHANGED', () => {
      const validationAction = 'error';

      it('updates the validationAction in state', (done) => {
        const unsubscribe = store.subscribe(() => {
          unsubscribe();
          expect(store.getState().validation.validationAction).to.equal(validationAction);
          done();
        });
        store.dispatch(validationActionChanged(validationAction));
      });
    });

    context('when the action is VALIDATION_LEVEL_CHANGED', () => {
      const validationLevel = 'moderate';

      it('updates the validationAction in state', (done) => {
        const unsubscribe = store.subscribe(() => {
          unsubscribe();
          expect(store.getState().validation.validationLevel).to.equal(validationLevel);
          done();
        });
        store.dispatch(validationLevelChanged(validationLevel));
      });
    });

    context('when the collection changes', () => {
      context('when there is no collection', () => {
        beforeEach(() => {
          store.onActivated(appRegistry);
          appRegistry.emit('collection-changed', 'db');
        });

        it('does not update the namespace in the store', () => {
          expect(store.getState().namespace).to.equal('');
        });

        it('resets the rest of the state to initial state', () => {
          expect(store.getState()).to.deep.equal({
            namespace: '',
            appRegistry: appRegistry,
            dataService: INITIAL_STATE.dataService,
            fields: INITIAL_STATE.fields,
            serverVersion: INITIAL_STATE.serverVersion,
            validation: INITIAL_STATE.validation,
            sampleDocuments: INITIAL_STATE.sampleDocuments,
            isZeroState: INITIAL_STATE.isZeroState,
            editMode: INITIAL_STATE.editMode
          });
        });
      });
    });

    context('when running in a readonly context', () => {
      beforeEach(() => {
        process.env.HADRON_READONLY = 'true';
        store.onActivated(appRegistry);
        appRegistry.emit('collection-changed', 'db');
      });

      afterEach(() => {
        process.env.HADRON_READONLY = 'false';
      });

      it('sets hardonReadOnly property as true', () => {
        expect(store.getState().editMode).to.deep.equal({
          collectionReadOnly: false,
          hardonReadOnly: true,
          writeStateStoreReadOnly: false,
          oldServerReadOnly: false
        });
      });
    });
  });
});
